;
; HUC_GFX.ASM  -  HuC Graphic Library
;

; ----
; local variables

          .zp
__spr_ptr .ds 2

          .bss
_satb    .ds 512	; the local SATB


; ----
; library code

	 .code

; vsync(char nb_frame)
; ----

_vsync:
	  lda	[__stack]
	  jmp	wait_vsync

; spr_set(char nb)
; ----
; load SI with the offset of the sprite to change
; SI = satb + 8 * sprite_number
; ----

_spr_set:
	  stz   <__spr_ptr+1
	  lda   [__stack]
	  asl   A
	  asl   A
	  asl   A
	  rol   <__spr_ptr+1
	  sta   <__spr_ptr
	  addw  #_satb,<__spr_ptr
	  rts

; spr_x(int value)
; ----

_spr_x:
	  ldy	#1
	  lda	[__stack],Y
	  tax
	  lda	[__stack]
	  ldy	#2
	  sta	[__spr_ptr],Y
	  txa
	  iny
	  sta	[__spr_ptr],Y
	  rts

; spr_y(int value)
; ----

_spr_y:
	  ldy	#1
	  lda	[__stack],Y
	  tax
	  lda	[__stack]
	  sta	[__spr_ptr]
	  txa
	  ldy	#1
	  sta	[__spr_ptr],Y
	  rts

; spr_pattern(int vaddr)
; ----

_spr_pattern:
	  ldy	#1
	  lda	[__stack],Y
	  sta	<__temp
	  lda	[__stack]
	  asl	A
	  rol	<__temp
	  rol	A
	  rol	<__temp
	  rol	A
	  rol	<__temp
	  rol	A
	  and	#$7
	  ldy	#5
	  sta	[__spr_ptr],Y
	  lda	<__temp
	  dey
	  sta	[__spr_ptr],Y
	  rts

; spr_ctrl(char mask, char value)
; ----

_spr_ctrl:
	  ldy	#2
	  lda	[__stack],Y
	  and	[__stack]
	  sta	<__temp
	  lda	[__stack]
	  ldy   #7
	  eor   #$FF
	  and   [__spr_ptr],Y
	  ora   <__temp
	  sta   [__spr_ptr],Y
	  rts

; spr_pal(char pal)
; ----

_spr_pal:
	  lda	[__stack]
	  and	#$0F
	  sta	<__temp
	  ldy	#6
	  lda	[__spr_ptr],Y
	  and	#$F0
	  ora	<__temp
	  sta   [__spr_ptr],Y
	  rts

; spr_pri(char pri)
; ----

_spr_pri:
	  lda	[__stack]
	  sta	<__temp
	  ldy	#6
	  lda	[__spr_ptr],Y
	  and	#$7F
	  ldx	<__temp
	  beq	.l1
	  ora	#$80
.l1:
	  sta	[__spr_ptr],Y
	  rts

; satb_update()
; ----

_satb_update:
	  update_satb _satb	; upload the SATB in VRAM
	  rts

; init_satb()
; ----

_init_satb:
	  init_satb _satb
	  rts

; get_color(int index)
; ----
; index: index in the palette (0-511)
; ----

_get_color:
	  lda	[__stack]
	  sta	$0402
	  ldy	#1
	  lda	[__stack],Y
	  sta	$0403
	  ldx	$0404
	  lda	$0405
	  rts


; set_color(int index, int color)
; ----
; set one palette entry to the specified color
; ----
; index: index in the palette (0-511)
; color: color value,  GREEN: bit 6-8
;                      RED:   bit 3-5
;                      BLUE:  bit 0-2
; ----

_set_color:
	  ldy	#2
	  lda	[__stack],Y
	  sta	$0402
	  iny
	  lda	[__stack],Y
	  sta	$0403
	  lda	[__stack]
	  sta	$0404
	  ldy	#1
	  lda	[__stack],Y
	  sta	$0405
	  rts

; set_color_rgb(int index, char r, char g, char b)
; ----
; set one palette entry to the specified color
; ----
; index: index in the palette (0-511)
; r:     red              RED:   bit 3-5
; g:     green            GREEN: bit 6-8
; b:     blue             BLUE:  bit 0-2
; ----

_set_color_rgb:
	  ldy	#6
	  lda	[__stack],Y
	  sta	$0402
	  iny
	  lda	[__stack],Y
	  sta	$0403

	  lda	[__stack]
	  and	#$7
	  sta	<__temp
	  ldy	#4
	  lda	[__stack],Y
	  asl	A
	  asl	A
	  asl	A
	  ora	<__temp
	  asl	A
	  asl	A
	  sta	<__temp
	  ldy	#2
	  lda	[__stack],Y
	  lsr	A
	  ror	<__temp
	  lsr	A
	  ror	<__temp
	  ldx	<__temp
	  stx	$0404
	  sta	$0405
	  rts

; set_palette(char block, int *data, char nb);
; ----
; set # palette block(s) of 16 color
; ----
; block: palette block index (0-31)
; data:  pointer to a 16-color block
; nb:    number of block to load (1-32)
; ----

_set_palette:
	  lda	[__stack]
	  sta	<_al
	  ldy	#2
	  lda	[__stack],Y
	  sta	<_si
	  iny
	  lda	[__stack],Y
	  sta	<_si+1
	  iny
	  lda	[__stack],Y
	  sta	<_cl
	  jmp	set_palette

