#include "bank.h"

static unsigned short offset_list[5] = {0xC000,
					0xA000,
                                        0x8000,
                                        0x6000,
                                        0x4000};

static int last_bank = 1;
/* We init this to 1 because we assume that bank 0 is reset bank
 * and bank 1 is auto set when beginning to parse the assembler file produced
 * by the C compiler
 */

static int last_offset = 0;
/* We init this to 0 because we assume that the first bank is scheduled to be
 * at offset $C000 which will be in position 0 of our table to correspond
 * this variable to a correct start offset for a bank (very confuse but easy
 * in fact)
 */

/*
 * check_org_error
 * This functions call the error function with an appropriate message if
 * the given value doesn't fit an org argument
 */
void check_org_error(int userorg)
{
  if (userorg % 0x2000)
     error("You should only use multiple of 0X2000");

  if ((userorg < 0X4000) || (userorg >= 0XE000))
     error("You mustn't use reserved bank");

  switch (userorg - (userorg % 0X2000))
    {
      case 0X0000:
         error("Bank reserved for I/O access");
         break;
      case 0X2000:
          error("Bank reserved for RAM");
          break;
      case 0XE000:
          error("Bank reserved for init and bios");
          break;
      default:
   }

 }

/*
 * newbank pseudo function to handle banking
 * without arg, change to the next bank using an offset so that there is less
 * chance to collide with previous or following (in fact, cycle between the
 * authorized offsets)
 * with one arg, change to the specified bank guessing the offset as previous
 * if the arg is less than 256, else consider it as the offset for a new bank
 * with two args, change to the bank specified by the first arg and set the
 * offset to the second arg
 * offset should be multiple of 0x2000, less than 0x10000
 * and shouldn't be 0x0000 (I/O page), 0x2000 (RAM page) nor 0xE000 (bios +
 * startup code)... 0x4000 should be used for sprite/bg tiles/colors. (but not
 * as compulsory as the 3 others pages)
 */
void donewbank(void)
{

  int userbank, userorg;

  if (!match("("))
    {
      error("missing (");
      ns();
      return;
    }

  if (match(")")) /* There's no arg */
    {
      last_bank++;
      last_offset++;
      last_offset &= 3; /* wrap around for the 4 possible
      			   automated generated pages */

      gbank(last_bank, offset_list[last_offset]);

      ns();
      return;
    }

  if (number(&userbank)) /* there's a valid arg there */
    {
     if (match(",")) /* there may be a second arg */
       {
        if (!number(&userorg))
          {
            error("incorrect ORG value");
            kill();
            return;
          }

        check_org_error(userorg);
        last_bank = userbank;
        last_offset = 6 - userorg / 0x2000;

        gbank(last_bank, userorg);

        }
     else
       {
         if (userbank>127) /* Then it's not a bank but an offset */
           {
            last_bank++;

            userorg = userbank;
            check_org_error(userorg);
            last_offset = 6 - userorg / 0x2000;

            }
         else
           {
             last_bank = userbank;
             last_offset++;
             last_offset &= 3;
            }

         gbank(last_bank, offset_list[last_offset]);

       }

     }
    else
       error("incorrect arg for newbank function");

  if (!match(")"))
      error("missing )");
  ns();
}
