/*
 * Debug.c
 *
 * Creation : 30 Mar '00
 * For use in HuC
 *
 * Provides some functions to print informations on screen
 * Uses lots of stuff from the sample1.asm demo source from the magic kit
 */

#define CHAR_BLK 36

#asm
        .bank  START_BANK
#endasm


/**[txh]********************************************************************

  Description: Initialize the debug font and such ...

  Return: nothing
  Example: init_debug_mode()   ^_^

***************************************************************************/


clr_debug_mode()
{
  #asm
	setvwaddr  $0		; set the VRAM address to $0000
        lda   #32               ; size 32 lines tall
clr_deb1:    ldx   #64               ; size 64 chars wide
        pha
clr_deb2:    cly

	; Fill each BAT map position with blank character

        stw   #CHAR(_offchar),video_data

        dex                     ; next block
        bne   clr_deb2
        pla
        dec   A                 ; next line
        bne   clr_deb1
   #endasm
}

init_debug_mode()
{
#asm
        map    _offchar          ; map in the memory bank
        ; load blank character into VRAM:
        vload  _offchar,#16

        set_bgpal  #0,_cgpal,#1  ; fill palette #0
; blank the background
#endasm

	clr_debug_mode();
#asm
	; load the characters for 0-9,A-F
	; into $1000 - $1100 in VRAM
	; (each character uses 16 words)

        ; 37 because I've temporary added the black character

        vload  _zero,#16*37


#endasm

        return;

}

/**[txh]********************************************************************

  Description: Put a character on screen

  Return: nothing
  Example: putdigit(0xa, 0x80) will draw a 'A' on the first line, at the first
           place.
           add 0x40 to go one line down and 1 for 1 character right

***************************************************************************/

putdigit(character, offset)
int character;
int offset;
{
  #asm
    vreg  #$00

    __ldwp __stack
    stx   video_data
    sta   video_data+1

     __addmi 2, __stack

     vreg  #$02

     __ldwp __stack
     __addwi 256

     stx   video_data
     ; lda   #$01
     sta   video_data+1

     __submi 2, __stack

  #endasm

}

/**[txh]********************************************************************

  Description: Display a decimal number the position of the RIGHT character
      is given in offset

  Return: nothing
  Example: putnumber(15, 0x81) will display the string 15 beginning at the
    very first character on screen (@ 0x80, don't forget)

***************************************************************************/

putnumber(number, offset)
int number;
int offset;
{
 do
 {
   putdigit(number%10, offset--);
   
   number = number / 10;
 }
 while (number);

 return;
}

/**[txh]********************************************************************

  Description: Display an ascii string on screen

  Return: nothing
  Example: putstring("hello world");
           only digits, lower case letters and spaces are handled

***************************************************************************/

putstring(string,offset)
char* string;
int offset;
{
  char actual_char;

  while (*string)
    {
      actual_char = *string++;

      if (actual_char == ' ')
         actual_char = CHAR_BLK;
      else
      if (actual_char<='9')
         actual_char -= 48; /* '0' = 48 */
      else
      if (actual_char>='a')
         actual_char -= 87; /* 'a' = 97,
                                and 'a' is represented by character 10 */

      putdigit(actual_char, offset++);
    }
  return;
}

newbank(0x4000);

/*
 * Blank char
 */
defchr(offchar,0x0FF0,0,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000);

/*
 * numbers from 0-9,A-F:
 */
defchr(zero,0x1000,0,
          0x00111110,
          0x01000011,
          0x01000101,
          0x01001001,
          0x01010001,
          0x01100001,
          0x00111110,
          0x00000000);

defchr(one,0x1010,0,
          0x00001000,
          0x00011000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00011100,
          0x00000000);

defchr(two,0x1020,0,
          0x00111110,
          0x01000001,
          0x00000001,
          0x00111110,
          0x01000000,
          0x01000000,
          0x01111111,
          0x00000000);

defchr(three,0x1030,0,
          0x00111110,
          0x01000001,
          0x00000001,
          0x00011110,
          0x00000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(four,0x1040,0,
          0x00000010,
          0x00100010,
          0x00100010,
          0x00111111,
          0x00000010,
          0x00000010,
          0x00000010,
          0x00000000);

defchr(five,0x1050,0,
          0x01111111,
          0x01000000,
          0x01000000,
          0x00111110,
          0x00000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(six,0x1060,0,
          0x00111110,
          0x01000000,
          0x01000000,
          0x01111110,
          0x01000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(seven,0x1070,0,
          0x00111111,
          0x00000001,
          0x00000001,
          0x00000010,
          0x00000100,
          0x00001000,
          0x00001000,
          0x00000000);

defchr(eight,0x1080,0,
          0x00111110,
          0x01000001,
          0x01000001,
          0x00111110,
          0x01000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(nine,0x1090,0,
          0x00111110,
          0x01000001,
          0x01000001,
          0x00111111,
          0x00000001,
          0x00000001,
          0x00111110,
          0x00000000);

defchr(ten,0x10A0,0,
          0x00111110,
          0x01000001,
          0x01000001,
          0x01111111,
          0x01000001,
          0x01000001,
          0x01000001,
          0x00000000);

defchr(eleven,0x10B0,0,
          0x01111110,
          0x01000001,
          0x01000001,
          0x01111110,
          0x01000001,
          0x01000001,
          0x01111110,
          0x00000000);

defchr(twelve,0x10C0,0,
          0x00111110,
          0x01000001,
          0x01000000,
          0x01000000,
          0x01000000,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(thirteen,0x10D0,0,
          0x01111110,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01111110,
          0x00000000);

defchr(fourteen,0x10E0,0,
          0x01111111,
          0x01000000,
          0x01000000,
          0x01111110,
          0x01000000,
          0x01000000,
          0x01111111,
          0x00000000);

defchr(fifteen,0x10F0,0,
          0x01111111,
          0x01000000,
          0x01000000,
          0x01111110,
          0x01000000,
          0x01000000,
          0x01000000,
          0x00000000);

defchr(char_g,0x1100,0,
          0x00111110,
          0x01000001,
          0x01000000,
          0x01000111,
          0x01000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(char_h,0x1110,0,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01111111,
          0x01000001,
          0x01000001,
          0x01000001,
          0x00000000);

defchr(char_i,0x1120,0,
          0x00011100,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00011100,
          0x00000000);

defchr(char_j,0x1130,0,
          0x00011110,
          0x00000100,
          0x00000100,
          0x00000100,
          0x00000100,
          0x01000100,
          0x00111000,
          0x00000000);

defchr(char_k,0x1140,0,
          0x01000010,
          0x01000100,
          0x01001000,
          0x01110000,
          0x01001000,
          0x01000100,
          0x01000010,
          0x00000000);

defchr(char_l,0x1150,0,
          0x01000000,
          0x01000000,
          0x01000000,
          0x01000000,
          0x01000000,
          0x01000000,
          0x01111110,
          0x00000000);

defchr(char_m,0x1160,0,
          0x01000001,
          0x01100011,
          0x01010101,
          0x01001001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x00000000);

defchr(char_n,0x1170,0,
          0x01000001,
          0x01100001,
          0x01010001,
          0x01001001,
          0x01000101,
          0x01000011,
          0x01000001,
          0x00000000);

defchr(char_o,0x1180,0,
          0x00111110,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(char_p,0x1190,0,
          0x01111110,
          0x01000001,
          0x01000001,
          0x01111110,
          0x01000000,
          0x01000000,
          0x01000000,
          0x00000000);

defchr(char_q,0x11A0,0,
          0x00111110,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000101,
          0x01000011,
          0x00111110,
          0x00000000);

defchr(char_r,0x11B0,0,
          0x01111110,
          0x01000001,
          0x01000001,
          0x01111110,
          0x01000100,
          0x01000010,
          0x01000001,
          0x00000000);


defchr(char_s,0x11C0,0,
          0x00111111,
          0x01000000,
          0x01000000,
          0x00111110,
          0x00000001,
          0x00000001,
          0x01111110,
          0x00000000);

defchr(char_t,0x11D0,0,
          0x01111111,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00000000);

defchr(char_u,0x11E0,0,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x00111110,
          0x00000000);

defchr(char_v,0x11F0,0,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x00100010,
          0x00100010,
          0x00011100,
          0x00000000);

defchr(char_w,0x1200,0,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01000001,
          0x01001001,
          0x01001001,
          0x00110110,
          0x00000000);

defchr(char_x,0x1210,0,
          0x01000001,
          0x00100010,
          0x00010100,
          0x00001000,
          0x00010100,
          0x00100010,
          0x01000001,
          0x00000000);

defchr(char_y,0x1220,0,
          0x01000001,
          0x00100010,
          0x00010100,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00001000,
          0x00000000);

defchr(char_z,0x1230,0,
          0x01111111,
          0x00000010,
          0x00000100,
          0x00001000,
          0x00010000,
          0x00100000,
          0x01111111,
          0x00000000);

defchr(char_blk,0x1240,0,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000,
          0x00000000);


/*
 *Simple palette entry
 *
 *entry #0 = black, #1-#15 are all white
 */

defpal(cgpal,   0x000,0x777,0x777,0x777,
                0x777,0x777,0x777,0x777,
                0x777,0x777,0x777,0x777,
                0x777,0x777,0x777,0x777);

#asm
        .bank  MAIN_BANK
#endasm
