;
; HUC.ASM  -  HuC Internal Library
;

; ----
; local variables

          .zp
__remain  .ds 2

          .code

          .mlist

; ----
; eq
; ----
; test egality of two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : word A:X equals 0 is the two args are egals
;       else non nul
; ----
; REMARK : signed compatible
; ----

                ; old version is 75 bytes long
                ; new version is 31 bytes long (58% shorter :)

eq:
   ldy #1       ; will return false by default
   sax
   cmp [__stack]
   bne .eq_end

   sax
   cmp [__stack],Y
   bne .eq_end

   cly          ; will return true

.eq_end:
   __addmi	2,__stack

   tya          ; if Y was 1, return A=X=0 -> false
   dec A        ; if Y was 0, return A=255 -> true

   clx

   rts

; TODO : it only handles unsigned stuff now, bou :(
; ----
; lt
; ----
; compare two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : word A:X is non nul if pushed word is strictly lower than
;       the word in A:X else nul
; ----
; REMARK : signed NON compatible
; ----

                ; old version is 111 bytes long
                ; new version is 37 bytes long (66% shorter :)

ult:    ; unsigned version, slow but works
lt:     ; signed version, doesn't work well

   ldy #1       ; false by default
   cmp [__stack],Y
   bcc .lt_end ; hibyte of the reg var < hibyte of the pushed var
   beq .lt_must_test_lobyte

   cly          ; set result to true
   bra .lt_end

.lt_must_test_lobyte
   sax
   cmp [__stack]
   bcc .lt_end       ; lobyte of the reg var < lobyte of the pushed var
   beq .lt_end
   cly               ; set result to true

.lt_end:
   __addmi      2,__stack

   tya          ; if Y was 1, return A=X=0 -> false
   dec A        ; if Y was 0, return A=255 -> true

   clx
   rts

; TODO : it only handles unsigned stuff now, bou :(
; ----
; ge
; ----
; compare two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : word A:X is non nul if pushed word is greater or egal to
;       the word in A:X else nul
; ----
; REMARK : signed NON compatible
; ----

uge:    ; unsigned version, works well
ge:     ; signed version, doesn't work well
    jsr lt
    inc A   ; assuming that true is represented by A = 255
    rts

; ----
; ne
; ----
; compare two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : word A:X is non nul if pushed word is diferent from
;       the word in A:X else nul
; ----
; REMARK : signed compatible
; ----

ne:
    jsr eq
    inc A   ; we're assuming that X = 0, and A = 0 if false (of course)
            ; but A = 255 if true
    rts


; TODO : it only handles unsigned stuff now, bou :(
; ----
; gt
; ----
; compare two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : word A:X is non nul if pushed word is strictly greater than
;       the word in A:X else nul
; ----
; REMARK : signed NON compatible
; ----

                ; old version is 109 bytes long
                ; new version is 35 bytes long (70% shorter :)

ugt:    ; unsigned version, slow but works
gt:     ; signed version, doesn't work well
   ldy #1
   cmp [__stack],Y
   beq .gt_must_test_lobyte
   bcs .gt_end ; hibyte of the reg var >= hibyte of the pushed var
   cly
   bra .gt_end

.gt_must_test_lobyte
   sax
   cmp [__stack]
   bcs .gt_end       ; lobyte of the reg var >= lobyte of the pushed var
   cly

.gt_end:
   __addmi      2,__stack
   tya
   dec A
   clx
   rts

; TODO : it only handles unsigned stuff now, bou :(
; ----
; le
; ----
; compare two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : word A:X is non nul if pushed word is lower or egal to
;       the word in A:X else nul
; ----
; REMARK : signed NON compatible
; ----

ule:    ; unsigned version, slow but works well
le:     ; signed version, doesn't work well
    jsr gt
    inc A       ; assuming that A=255 if true
    rts


; ----
; asl
; ----
; shift the pushed word left by the register word
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : Register word egals the previous pushed value
;       shifted left by A:X
; ----
; REMARK : only the lower byte of the right operand is taken in account
;          signed compatible
; ----
asl:
   stx __temp
   __ldwp __stack
   ldy __temp
   beq .asl_end
   sta __temp
   sax

.asl_begin
   asl a
   rol __temp
   dey
   bne .asl_begin

   sax
   lda __temp
.asl_end
   __addmi 2, __stack
   rts


; ----
; asr
; ----
; shift the pushed word right by the register word
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : Register word egals the previous pushed value
;       shifted right by A:X
; ----
; REMARK : only the lower byte of the right operand is taken in account
;          signed compatible
; ----
asr:
   stx __temp
   __ldwp __stack
   ldy __temp
   beq .asr_end
   sta __temp
   sax

.asr_begin
   cpx #$80
   ror __temp
   ror a

   dey
   bne .asr_begin

   sax
   lda __temp
.asr_end
   __addmi 2, __stack
   rts

; TODO : optimize .... make it available for signed stuff
; ----
; smul
; ----
; multiply the two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : Register word egals the previous pushed value
;       multiplied by A:X
; ----
; REMARK : signed NON compatible
; ----

smul:
  __stw _bx
  __ldwp __stack
  __stw _ax
  jsr mulu16
  __addmi 2, __stack
  __ldw _cx
  rts


; TODO : optimize .... make it available for signed stuff
; ----
; sdiv
; ----
; divide the two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : Register word egals the previous pushed value
;       divided by A:X
; ----
; REMARK : signed NON compatible
; ----

sdiv:
        __stw   __ptr
        __ldwp  __stack
        __stw   __temp

        lda	#0
 	sta	__remain+1
 	ldy	#16
.sdiv_begin:	asl	__temp
 	rol	__temp+1
 	rol	a
 	rol	__remain+1
 	pha
 	cmp	__ptr
 	lda	__remain+1
 	sbc	__ptr+1
	bcc	.sdiv_end
	sta	__remain+1
	pla
	sbc	__ptr
	pha
	inc	__temp
.sdiv_end:	pla
	dey
	bne	.sdiv_begin
	sta	__remain

        __addmi 2, __stack
        __ldw __temp

	rts


; TODO : optimize .... make it available for signed stuff
; ----
; smod
; ----
; give the integer remainder of the two words
; ----
; IN :  First word on the C stack
;       Another word in A:X
; ----
; OUT : Register word egals the remainder of the division of the
;       pushed value by A:X
; ----
; REMARK : signed NON compatible
; ----

smod:
        __stw   __ptr
        __ldwp  __stack
        __stw   __temp

        lda	#0
 	sta	__remain+1
 	ldy	#16
.smod_begin:	asl	__temp
 	rol	__temp+1
 	rol	a
 	rol	__remain+1
 	pha
 	cmp	__ptr
 	lda	__remain+1
 	sbc	__ptr+1
	bcc	.smod_end
	sta	__remain+1
	pla
	sbc	__ptr
	pha
	inc	__temp
.smod_end:	pla
	dey
	bne	.smod_begin
	sta	__remain

        __addmi 2, __stack
        __ldw __remain

	rts

