/*	File opt.c: 2.1 (83/03/20,16:02:09) */
/*% cc -O -c %
 *
 */

#include <stdio.h>
#include "defs.h"
#include "data.h"

/* defines */
#define Q_SIZE		8

/* optimized instructions */
#define X_LDB_MASK	0xFFFF0
#define X_LDB		0x10000
#define X_LDB_A		0x10000
#define X_LDB_S		0x10001
#define X_LDB_P		0x10002

#define X_PEA_MASK	0xFFFF0
#define X_PEA		0x10010
#define X_PEA_A		0x10010
#define X_PEA_S		0x10011

/* typedefs */
typedef struct {
	int ins;
	int type;
	int data;
} INS;

/* locals */
static INS q_ins[Q_SIZE];
static int q_rd;
static int q_wr;
static int q_nb;


/* ----
 * push_ins()
 * ----
 *
 */

void push_ins(int ins, int type, int data)
{
	/* optimization level 1 - simple peephole optimizer,
	 * replace known instruction patterns by highly
	 * optimized asm code
	 */
	if (optimize >= 1)
	{
		INS *p[Q_SIZE];
		int  i, j;
		int  flag = 0;

		/* check queue size */
		if (q_nb == Q_SIZE)
		{
			/* queue is full - flush the last instruction */
			gen_code(q_ins[q_rd].ins,
					 q_ins[q_rd].type,
					 q_ins[q_rd].data);

			/* advance queue read pointer */
			q_rd++;
			q_nb--;

			if (q_rd == Q_SIZE)
				q_rd = 0;
		}

		/* push new instruction */
		q_ins[q_wr].ins  = ins;
		q_ins[q_wr].type = type;
		q_ins[q_wr].data = data;

		q_wr++;
		q_nb++;

		if (q_wr == Q_SIZE)
			q_wr = 0;

	lv1_loop:
		/* precalculate instruction pointer */
		for (i = 0, j = q_wr; i < q_nb; i++)
		{
			j -= 1;
			if (j < 0)
				j += Q_SIZE;
			p[i] = &q_ins[j];
		}

		/* LEVEL 1 FUN STUFF STARTS HERE */

		/* 3-instruction patterns */
		if (q_nb >= 3)
		{
			/*  __ldw   __stack             --> @_pea_s	i
			 *  __addwi i
			 *  __pushw
			 *
			 *  ====
			 *  bytes  : 4+ 7+23 = 34       --> 34
			 *  cycles : 8+10+49 = 67       --> 67
			 *
			 */
			if ((p[0]->ins  == I_PUSHW) &&
				(p[1]->ins  == I_ADDWI) &&
				(p[2]->ins  == I_LDW) &&
	
				(p[2]->type == T_STACK))
			{
				/* replace code */
				p[2]->ins  = X_PEA_S;
				p[2]->data = p[1]->data;
				flag = 1;
			}

			/*  __stw   __ptr               --> @_ldb_p
			 *  __ldbp  __ptr
			 *  __extw
			 *
			 *  ====
			 *  bytes  : 4+3+ 7 = 14        --> 14
			 *  cycles : 8+9+10 = 27        --> 27
			 *
			 */
			else if
			   ((p[0]->ins  == I_EXTW) &&
				(p[1]->ins  == I_LDBP) &&
				(p[2]->ins  == I_STW) &&
	
				(p[2]->type == T_PTR))
			{
				/* replace code */
				p[2]->ins  = X_LDB_P;
				flag = 1;
			}

			/*  __ldw   __stack             --> @_ldb_s i
			 *  __addwi i
			 *  @_ldb_p
			 *
			 *  ====
			 *  bytes  : 4+ 7+14 = 25       --> 10
			 *  cycles : 8+10+27 = 45       --> 19
			 *
			 */
			else if
			   ((p[0]->ins  == X_LDB_P) &&
				(p[1]->ins  == I_ADDWI) &&
				(p[2]->ins  == I_LDW) &&
	
				(p[2]->type == T_STACK))
			{
				/* replace code */
				p[2]->ins  = X_LDB_S;
				p[2]->data = p[1]->data;
				flag = 1;
			}

			/* flush queue */
			if (flag)
			{
				flag  = 0;
				q_wr -= 2;
				q_nb -= 2;

				if (q_wr < 0)
					q_wr += Q_SIZE;

				/* loop */
				goto lv1_loop;
			}			
		}
	}

level_2:
	/* optimization level 2 - ??? */
	if (optimize >= 2)
	{



	}
}


/* ----
 * flush_ins()
 * ----
 * flush instruction queue
 *
 */

void flush_ins()
{
	while (q_nb)
	{
		/* gen code */
		gen_code(q_ins[q_rd].ins,
				 q_ins[q_rd].type,
				 q_ins[q_rd].data);

		/* advance and wrap queue read pointer */
		q_rd++;
		q_nb--;

		if (q_rd == Q_SIZE)
			q_rd = 0;
	}

	/* reset queue */
	q_rd = 0;
	q_wr = 0;
	q_nb = 0;
}


/* ----
 * gen_asm()
 * ----
 * generate optimizer asm code
 * 
 */

gen_asm(int ins, int type, int data)
{
	switch (ins) {
	case X_LDB_P:
		ol("@_ldb_p");
		break;

	case X_LDB_S:
		ot("@_ldb_s\t");
		outdec(data);
		nl();
		break;

	case X_PEA_S:
		ot("@_pea_s\t");
		outdec(data);
		nl();
		break;
	}
}

